/**
 * Manage the asterisk events.
 *
 * @module ast_proxy
 * @submodule plugins_event_13
 */

/**
 * The module identifier used by the logger.
 *
 * @property IDLOG
 * @type string
 * @private
 * @final
 * @readOnly
 * @default [shutdown]
 */
var IDLOG = '[shutdown]';

/**
 * The asterisk proxy.
 *
 * @property astProxy
 * @type object
 * @private
 */
var astProxy;

(function() {

  /**
   * The logger. It must have at least three methods: _info, warn and error._
   *
   * @property logger
   * @type object
   * @private
   * @default console
   */
  var logger = console;

  try {
    /**
     * The plugin that handles the shutdown event.
     *
     * @class shutdown
     * @static
     */
    var shutdown = {
      /**
       * It is called from _astproxy_ component for each shutdown event
       * received from the asterisk.
       *
       * @method data
       * @param {object} data The asterisk event data
       * @static
       */
      data: function(data) {
        try {
          if (data && data.event === 'Shutdown') {
            logger.info(IDLOG, 'received event ' + data.event);
            astProxy.proxyLogic.evtAstShutdown({
              shutdown: data.shutdown,
              restart: data.restart
            });
          } else {
            logger.warn(IDLOG, 'shutdown event not recognized');
          }
        } catch (err) {
          logger.error(IDLOG, err.stack);
        }
      },

      /**
       * Set the logger to be used.
       *
       * @method setLogger
       * @param {object} log The logger object
       * @static
       */
      setLogger: function(log) {
        try {
          if (typeof log === 'object' &&
            typeof log.info === 'function' &&
            typeof log.warn === 'function' &&
            typeof log.error === 'function') {

            logger = log;
          } else {
            throw new Error('wrong logger object');
          }
        } catch (err) {
          logger.error(IDLOG, err.stack);
        }
      },

      /**
       * Store the asterisk proxy to visit.
       *
       * @method visit
       * @param {object} ap The asterisk proxy module.
       */
      visit: function(ap) {
        try {
          // check parameter
          if (!ap || typeof ap !== 'object') {
            throw new Error('wrong parameter');
          }
          astProxy = ap;
        } catch (err) {
          logger.error(IDLOG, err.stack);
        }
      }
    };

    // public interface
    exports.data = shutdown.data;
    exports.visit = shutdown.visit;
    exports.setLogger = shutdown.setLogger;

  } catch (err) {
    logger.error(IDLOG, err.stack);
  }
})();
